import os.path
from configparser import ConfigParser
from migration_machine.utils.exceptions import ConfigException

MODELLING = "modelling"
MERGER = "merger"
MAPPER = "mapper"
CONVERTER = "converter"
PARSER_TARGET = "parser_target"
PARSER_SOURCE = "parser_source"
DOMAIN = "domain"
SETTINGS = "settings"

CONFIG_DOMAIN_SOURCE_FOLDER = "SOURCE"
CONFIG_DOMAIN_TARGET_FOLDER = "TARGET"


class Config:
    def __init__(self, config_file):
        self.modelling = None
        self.merger = None
        self.mapper = None
        self.converter = None
        self.parser_target = None
        self.parser_source = None
        self.domain = None
        self.settings = None
        self.sections = None
        self.parser = None
        self.file = config_file
        self.load_sections()
        self.read_configuration()
        self.load_config()

    def load_sections(self):
        if "transform" in os.path.basename(self.file):
            self.sections = [SETTINGS, DOMAIN, PARSER_SOURCE, PARSER_TARGET, MAPPER, MERGER]
        else:
            self.sections = [SETTINGS, DOMAIN, CONVERTER, MODELLING]

    def load_config(self):
        if "transform" in os.path.basename(self.file):
            self.settings = self.get_config(SETTINGS)
            self.domain = self.get_config(DOMAIN)
            self.parser_source = self.get_config(PARSER_SOURCE)
            self.parser_target = self.get_config(PARSER_TARGET)
            self.mapper = self.get_config(MAPPER)
            self.merger = self.get_config(MERGER)
        else:
            self.settings = self.get_config(SETTINGS)
            self.domain = self.get_config(DOMAIN)
            self.converter = self.get_config(CONVERTER)
            self.modelling = self.get_config(MODELLING)

    def read_configuration(self):
        self.parser = ConfigParser()
        self.parser.read(self.file)
        sections = self.parser.sections()
        self.validate_sections(sections)

    def validate_sections(self, sections):
        difference = list(set(self.sections) - set(sections))
        if len(difference) > 0:
            raise ConfigException("Missing configuration sections {}".format(difference))

    def get_config(self, type):
        return {
            item[0]: item[1]
            for item in self.parser.items(type)
            }
