# Mapping processor following the parser definition
import json
import argparse
import sys

import logging

import os
from frictionless import Resource
import petl
from shutil import copyfile


def merge_table(table, path, pk, fields, dir, output):
    file_pre = path[path.rfind('/') + 1:] if path.rfind('/') >= 0 else path
    onlyfiles = [f for f in os.listdir(dir) if os.path.isfile(os.path.join(dir, f)) and f.find(file_pre) == 0]
    if len(onlyfiles) == 0: return

    os.makedirs(os.path.dirname(os.path.abspath(output + '/' + path)), exist_ok=True)

    if len(onlyfiles) == 1:
        t = petl.fromcsv(dir + '/' + onlyfiles[0])
        rename_headers(t)
        t = normalize_table(t, fields)
        petl.tocsv(t, output + '/' + path)
        return

    merged = None
    for f in onlyfiles:
        print(f)
        t = petl.fromcsv(dir + '/' + f)
        if petl.nrows(t) == 0: continue
        t = rename_headers(t)

        if merged is None: merged = t
        else:
            merged = petl.cat(merged, t)
            conflicts = petl.conflicts(merged, pk, cache=False)
            if petl.nrows(conflicts) > 0:
                print('CONFLICTS found for ' + path)
                print(conflicts)
                raise Exception('CONFLICTS found for ' + path)
            merged = petl.mergeduplicates(merged, pk)

    merged = normalize_table(merged, fields)
    petl.tocsv(merged, output + '/' + path)
    return

def rename_headers(table):
    for col in [*petl.header(table)]:
        if col[0:2] == 't_':
            table = petl.rename(table, col, col[2:])
    return table

def normalize_table(table, fields):
    h = petl.header(table)
    for i in range(len(fields)):
        f = fields[i]['name']
        if f in h:
            table = petl.movefield(table, f, i)
        else:
            table = petl.addfield(table, f, index=i)
    return table

############################# MAIN
parser = argparse.ArgumentParser(description='Merge dapapackage files')
parser.add_argument('-p', '--package', help='Package specification file')
parser.add_argument('-d', '--dir', default='.', help='working data directory')
parser.add_argument('-o', '--output', default='./output', help='output directory')
parser.add_argument('-t', '--table', default=None, help='the table to map')

args = vars(parser.parse_args())

spec_file = args['package']
output = args['output']
dir = args['dir']
table = args["table"]

with open(spec_file) as f: spec = json.load(f)

for r in spec['resources']:
    if table != None and r['name'] == table:
        merge_table(table, r['path'], r['schema']['primaryKey'], r['schema']['fields'], dir, output)
        break
    elif table == None:
        merge_table(r['name'], r['path'], r['schema']['primaryKey'], r['schema']['fields'], dir, output)
