import sqlite3
import pandas as pd
from migration_machine.utils.utils import LoggingUtils
import migration_machine.config as configurations

"""
Using SQLite as data storage for Pandas.
SQLite is a fully-featured relational database that runs as library, not a server; Python ships with built-in support.
SQLite stores its data in a single file. Instead of having to manage one CSV file, you have to manage one SQLite database file.
"""


def read_from_sqlite(data_dir, logging):
    logging.logging_util.log_generator("Starting to read_from_sqlite...", True)
    conn = sqlite3.connect("{}/sqlite/{}".format(data_dir, "A01_A01.sqlite"))
    q = "SELECT codice_anagrafe_generale,tipo_controparte  FROM A01_A01 WHERE tipo_controparte <= '11999' and tipo_controparte >= '11000'"
    df = pd.read_sql_query(q, conn)
    logging.logging_util.log_generator("Ending read_from_sqlite", True)
    return df


def read_from_file(data_dir, logging):
    logging.logging_util.log_generator("Starting to read_from_file...", True)
    df = pd.concat(
        df.query("tipo_controparte <= '11999' and tipo_controparte >= '11000'") for df in
        pd.read_csv("{}/csv/{}".format(data_dir, "A01_A01.csv"), chunksize=500000,
                    usecols=["codice_anagrafe_generale", "tipo_controparte"], encoding="ISO-8859-1", sep=",", dtype=str)
        )
    logging.logging_util.log_generator("Ending read_from_file...", True)
    return df


def load_sqlite():
    db = sqlite3.connect("{}/sqlite/{}.sqlite".format(data_dir, data_dir))
    # Load the csv in chunks
    chunks = pd.read_csv("{}/csv/{}".format(data_dir, "A01_A01.csv"), sep=",", dtype=str, chunksize=500000)
    for chunk in chunks:
        chunk.to_sql("A01_A01", db, if_exists="append")

    # creating index allowing us to quickly search the database using these columns
    db.execute("CREATE INDEX tipo_controparte on A01_A01(tipo_controparte)")
    db.execute("CREATE INDEX stato_controparte on A01_A01(stato_controparte)")
    db.close()


if __name__ == "__main__":
    configs = configurations.Config("examples/SIB2SICRA/convert.ini")
    logging = LoggingUtils(configs)
    data_dir = configs.domain["domain_data_path"]
    read_from_sqlite(data_dir, logging)
    read_from_file(data_dir, logging)
