import os.path
from datetime import datetime
import logging
from logging.handlers import TimedRotatingFileHandler


class LoggingUtils:
    os.makedirs("logs", exist_ok=True)
    formatter = logging.Formatter("%(asctime)s - %(levelname)s - %(message)s")
    handler = TimedRotatingFileHandler('logs/migration.log', when="midnight", interval=1, encoding='utf8')
    handler.suffix = "%Y-%m-%d"
    handler.setFormatter(formatter)
    logger = logging.getLogger()
    logger.addHandler(handler)

    def __init__(self, config):
        self.last_time = datetime.now()
        self.last_time_doc = datetime.now()
        os.makedirs("logs", exist_ok=True)
        self.__class__.logger.setLevel(eval(config.settings["logging_level"]))

    def log_generator(self, message, difference=True):
        """
        Method to format the log generation for time tracking of each method execution
        :param difference: choose to
        :param message: Contains the message to be shown on logs
        :return:
        """
        if (difference):
            diff = datetime.now() - self.last_time
            self.logger.info("difference in time from the last action: %s : %s", str(diff), message)
            self.last_time = datetime.now()
        else:
            self.logger.info("%s", message)

    def log_generator_error(self, message, difference=True):
        """
        Method to format the log generation for time tracking of each method execution
        :param difference: choose to
        :param message: Contains the message to be shown on logs
        :return:
        """
        if (difference):
            diff = datetime.now() - self.last_time
            self.logger.error("difference in time from the last action: %s : %s", str(diff), message)
            self.last_time = datetime.now()
        else:
            self.logger.error("%s", message)

    def log_documents(self, name, iter):
        """
        Method to save logs into mongo so that we can order the execution time of each entity parser
        :param name:
        :param iter:
        """
        diff = (datetime.now() - self.last_time_doc).seconds

        client = MongoClient(port=27017)  # "mongodb://127.0.0.1:27017"
        db = client.migrationLogs
        log = {
            'name': name,
            'iter': iter,
            'diff_time': diff,
            'total_sec': (datetime.now() - self.last_time_doc).total_seconds()
            }
        db.entity_logs.insert_one(log)
        self.last_time_doc = datetime.now()


def normalize_field_name(original):
    """
    Normalize field names by removing special chars
    :param original:
    :return:
    """
    return original.lower().replace(' ', '_').replace('-', '_').replace('/', '') \
        .replace('.', '').replace('"', '').replace(',', '').replace('\'', '').replace('?', '') \
        .replace('*', '').replace('(', '').replace(')', '').replace('à', 'a').replace('ù', 'u')
